#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include "cat.h"

#include "cb.h"
#include "ui.h"
#include "search.h"
#include "savecat.h"
#include "utils.h"

extern GdkPixmap *img_dir;
extern GdkBitmap *img_dir_mask;
extern GdkPixmap *img_file;
extern GdkBitmap *img_file_mask;

static void delete_disk_cb(Ui *ui, Catalog *cat);

void dialog_keypress_cb(GtkWidget *widget, GdkEventKey *key, GtkWidget *window)
{
    /* GTK really sucks.  This is what we have to do to get normal
     * user accepted behavior of pressing "esc" key to get rid of a dialog
     * box */
    if (key->keyval == GDK_Escape)
	gtk_widget_destroy(window);
}

void menu_cb(gpointer data, int id, GtkWidget *item)
{
    Ui *ui = (Ui *)data;
    Catalog *cat = ui->cat;

    switch (id) {
	case 11:		/* file-save */
	    update_statusbar(ui, SB_SAVING);
	    savecat(cat); 
	    update_statusbar(ui, SB_READY);
	    break;
	case 19:		/* file-exit */
	    gtk_main_quit();
	    break;
	case 21:
	    new_add_cd(ui);
	    break;
	case 22:
	    new_add_path(ui);
	    break;
	case 23:
	    new_find(ui);
	    break;
	case 31:		/* catalog-delete disk */
	    delete_disk_cb(ui, cat);
	    break;
	default:
	    break;
    }
}

static int calc_dir_size(GNode *node, gpointer data)
{
    Item *item = ITEM(node->data);
    if (item && (item->size < 0xfffffffe))
	*(unsigned int *)data += item->size;
    return FALSE;
}

static int filelist_fill(GNode *node, gpointer data)
{
    Ui *ui = (Ui *)data;
    Item *item = ITEM(node->data);
    GtkCList *clist = GTK_CLIST(ui->filelist);
    int row;

    /* skip root node and skip "current dir" node */
    if (item && (node != ui->node)) {
	char *list[] = { NULL, NULL, NULL, NULL };

	/* prepare the column data */
	if (item->size < 0xfffffffe) {		/* normal file or dir */
	    unsigned int size = item->size;
	    if (!item->is_leaf) {
		/* this is a dir, calculate its size */
		g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1, calc_dir_size, &size);
	    }
	    list[1] = (size > 10 << 20) ? 
		g_strdup_printf("%d Mb", size >> 20) :
		g_strdup_printf("%d", size);
	    list[2] = datetime(item->time);

	    /* add the row */
	    row = gtk_clist_append(clist, list);

	    if (item->is_leaf) {	/* its a file */
		gtk_clist_set_pixtext(clist, row, 0, item->name,
			UI_SPACING, img_file, img_file_mask);
	    } else {			/* its a dir */
		gtk_clist_set_pixtext(clist, row, 0, item->name,
			UI_SPACING, img_dir, img_dir_mask);
	    }
	    g_free(list[2]);
	    g_free(list[1]);
	}
    }
    return FALSE;
}

static void update_filelist(GtkCTree *ctree, GtkCTreeNode *node, Ui *ui)
{
    Disk *disk;
    GNode *gnode;

    /* top-level node has the disk pointer */
    if (GTK_CTREE_ROW(node)->level == 1) {
	disk = gtk_ctree_node_get_row_data(ctree, node);
	gnode = disk->tree;
    } else {		/* all sequential nodes have gnode pointer */
	gnode = gtk_ctree_node_get_row_data(ctree, node);
    }

    /* so that we skip the current name */
    ui->node = gnode;

    gtk_clist_freeze(GTK_CLIST(ui->filelist));
    gtk_clist_clear(GTK_CLIST(ui->filelist));
    g_node_traverse(gnode, G_PRE_ORDER, G_TRAVERSE_ALL, 2, filelist_fill, ui);
    gtk_clist_sort(GTK_CLIST(ui->filelist));
    gtk_clist_thaw(GTK_CLIST(ui->filelist));
    gtk_clist_select_row(GTK_CLIST(ui->filelist), 0, 0);
}

void cb_dirtree_select_row(GtkCTree *ctree, GtkCTreeNode *node, int column,
	Ui *ui)
{
    update_filelist(ctree, node, ui);
}

void cb_dirtree_expand_row(GtkCTree *ctree, GtkCTreeNode *node,
	Ui *ui)
{
    /* gtk_ctree_select(ctree, node);
       update_filelist(ctree, node, ui); */
}

void cb_dirtree_collapse_row(GtkCTree *ctree, GtkCTreeNode *node,
	Ui *ui)
{
    /* gtk_ctree_select(ctree, node);
       update_filelist(ctree, node, ui); */
}

static void delete_disk_cb(Ui *ui, Catalog *cat)
{
    GtkCTree *ctree;
    GtkCList *clist;
    GtkCTreeNode *node;
    Disk *disk;

    ctree = GTK_CTREE(ui->dirtree);
    clist = GTK_CLIST(ctree);
    node = gtk_ctree_node_nth(ctree, clist->focus_row);
    if (node == NULL) {
	g_print("select a disk, fewl\n");
	return;
    }
    if (GTK_CTREE_ROW(node)->parent != NULL) {
	g_print("that's not a disk, fewl\n");
	return;
    }
    disk = gtk_ctree_node_get_row_data(ctree, node);
    nuke_disk(disk, NULL);
    cat->disks = g_list_remove(cat->disks, disk);
    gtk_ctree_remove_node(ctree, node);
}

void add_cd_entry_cb(GtkWidget *widget, GtkWidget *window)
{
    Ui *ui;
    GtkWidget *label;
    char *name;

    ui = gtk_object_get_data(GTK_OBJECT(window), "ui");
    label = gtk_object_get_data(GTK_OBJECT(window), "label");

    if (GTK_ENTRY(label)->text_length == 0)
	return;

    name = gtk_editable_get_chars(GTK_EDITABLE(label), 0, -1);

    gtk_widget_destroy(window);

    system("mount /cdrom");
    cat_add_disk(ui, name, "/cdrom");
    system("umount /cdrom");

    update_statusbar(ui, SB_EJECTING);

    /* let GTK update the UI */
    while (gtk_events_pending())
	gtk_main_iteration();
    
    system("eject");
    update_statusbar(ui, SB_READY);

    g_free(name);
}

void find_entry_cb(GtkWidget *widget, GtkWidget *window)
{
    Ui *ui;
    GtkWidget *entry;
    GtkWidget *toggle;
    GtkWidget *option;
    Search *search;
    
    ui = gtk_object_get_data(GTK_OBJECT(window), "ui");
    entry = gtk_object_get_data(GTK_OBJECT(window), "entry");
    toggle = gtk_object_get_data(GTK_OBJECT(window), "toggle");
    option = gtk_object_get_data(GTK_OBJECT(window), "option");

    if (GTK_ENTRY(entry)->text_length == 0)
	return;

    search = g_new0(Search, 1);
    search->ui = ui;
    search->pattern = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
    search->sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
    search->type = COMPARE_GLOB;

    update_statusbar(ui, SB_SEARCHING);

    find_file(search);

    g_free(search->pattern);
    g_free(search);
    update_statusbar(ui, SB_READY);
}

void add_path_entry_cb(GtkWidget *widget, GtkWidget *window)
{
    Ui *ui;
    GtkWidget *entry;
    GtkWidget *label;

    char *path;
    char *name;

    ui = gtk_object_get_data(GTK_OBJECT(window), "ui");
    entry = gtk_object_get_data(GTK_OBJECT(window), "entry");
    label = gtk_object_get_data(GTK_OBJECT(window), "label");

    if (GTK_ENTRY(entry)->text_length == 0)
	return;
    if (GTK_ENTRY(label)->text_length == 0)
	return;

    path = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
    name = gtk_editable_get_chars(GTK_EDITABLE(label), 0, -1);

    gtk_widget_destroy(window);

    cat_add_disk(ui, name, path);
    g_free(name);
    g_free(path);
}
