#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "cat.h"

#include "loadcat.h"

static int load_disk(GNode *root, int fd)
{
    Item *item = NULL;
    GNode *node = NULL;
    int reset = TRUE;

    while (reset) {
	item = g_new0(Item, 1);
	read(fd, item, sizeof(Item));
	
	
	if (item->size == 0xfffffffe) {		/* end of directory, continue */
	    reset = FALSE;
	} else if (item->size == 0xffffffff) {	/* end of disk reached, return */
	    g_free(item);
	    return FALSE;
	}

	item->name = g_new0(char, item->namelen);
	read(fd, item->name, item->namelen);

	if (item->size != 0xffffffff) {
	    node = g_node_new(item);
	    g_node_append(root, node);

	    /* recurse into the dir */
	    if (!item->is_leaf)
		load_disk(node, fd);

	} else {
	    g_free(item->name);
	    g_free(item);
	}
    }
    return TRUE;
}

Catalog *loadcat(const char *filename)
{
    Catalog *cat = NULL;
    int fd;
    char catid[9];
    int n_disks;
    int done = FALSE;

    cat = g_new0(Catalog, 1);
    cat->catfile = g_strdup(filename);

    fd = open(cat->catfile, O_RDONLY, 0644);
    if (fd == -1)
	return cat;

    read(fd, catid, 9);
    read(fd, &n_disks, sizeof(unsigned int));

    while (!done && n_disks--) {
	Disk *disk;
	unsigned char diskid[4];

	read(fd, diskid, 4);
	/* g_print("DiskID: %02x%02x%02x%02x\n", diskid[0], diskid[1], diskid[2], diskid[3]); */
	/* XXX check */

	disk = g_new0(Disk, 1);
	cat->disks = g_list_append(cat->disks, disk);
	read(fd, disk->label, 20);
	/* g_print("Disklabel: %s\n", disk->label); */
	read(fd, &disk->n_dirs, sizeof(unsigned int));
	read(fd, &disk->n_files, sizeof(unsigned int));
	read(fd, catid, 8);

	disk->tree = g_node_new(NULL);
	done = load_disk(disk->tree, fd);
    }

    close(fd);
    return cat;
}
