#include <gtk/gtk.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "cat.h"

#include "scan.h"
#include "utils.h"

extern unsigned int total;

Item *new_item(const char *name, const size_t size,
	       const time_t time, const int level, const int folder)
{
    Item *item;

    item = g_new0(Item, 1);

    item->size = size;
    item->time = time;
    item->is_leaf = folder ? 0 : 1;
    item->level = level;

    item->namelen = strlen(name) + 1;
    item->name = g_strdup(name);

    return item;
}

unsigned int scan_tree(GNode *root, const char *path, int reset)
{
    struct dirent *entry;
    struct stat info;
    DIR *dir;
    static int level;
    
    char *name;
    char *fullpath;
    Item *item = NULL;
    GNode *node = NULL;

    if (reset)
	level = 1;

    fullpath = g_strconcat(path, "/", NULL);

    if ((dir = opendir(fullpath)) == NULL) {
	g_warning("Cannot scan %s\n", path);
	return FALSE;
    }

    while ((entry = readdir(dir))) {
	/* ignore . and .. */
	if (strcmp(entry->d_name, ".") &&
	    strcmp(entry->d_name, "..")) {
	    
	    /* get full pathname */
	    name = g_strconcat(fullpath, entry->d_name, NULL);
	    if (lstat(name, &info) == -1) {		/* unreadable */
		item = new_item("!unreadable!", 0, 0, level, FALSE);
		g_node_append(root, g_node_new(item));
	    } else if (S_ISLNK(info.st_mode)) {		/* symlink */
		char *linkname;
		char *fullname;

		linkname = readlink_alloc(name);
		fullname = g_strconcat(entry->d_name, " -> ", linkname, NULL);
		item = new_item(fullname, 0, info.st_ctime, level, FALSE);
		g_node_append(root, g_node_new(item));
		g_free(fullname);
		g_free(linkname);
	    } else if (S_ISDIR(info.st_mode)) {		/* directory */
		char *newdir;

		/* let GTK update the UI */
		while (gtk_events_pending())
		    gtk_main_iteration();

		item = new_item(entry->d_name, 0,
				info.st_ctime, level, TRUE);
		node = g_node_new(item);
		g_node_append(root, node);

		/* recurse into the dir */
		newdir = g_strconcat(path, "/", entry->d_name, NULL);
		level++;
		scan_tree(node, newdir, FALSE);

		/* end of directory mark */
		item = new_item("..", 0xfffffffe, 0, level, FALSE);
		node = g_node_new(item);
		g_node_append(root, node);
		level--;
		g_free(newdir);
	    } else {					/* filename */
		item = new_item(entry->d_name, info.st_size,
				info.st_ctime, level, FALSE);
		g_node_append(root, g_node_new(item));
	    }
	    g_free(name);
	}
    }
    closedir(dir);
    g_free(fullpath);
    return TRUE;
}
